import React, { Fragment } from 'react'
import { get } from 'lodash'
import styled from 'styled-components'
import { reduxForm } from 'redux-form'
import { th } from '@pubsweet/ui-toolkit'
import { required } from 'xpub-validators'
import { compose, setDisplayName, withHandlers, withProps } from 'recompose'
import {
  H2,
  Menu,
  Button,
  Spinner,
  TextField,
  ValidatedField,
} from '@pubsweet/ui'
import {
  Row,
  Text,
  Item,
  Label,
  IconButton,
  ModalMenuItem,
  RowOverrideAlert,
} from 'pubsweet-component-faraday-ui'

const FormModal = ({
  roles,
  title,
  titles,
  onClose,
  subtitle,
  onConfirm,
  countries,
  isFetching,
  confirmText = 'OK',
  cancelText = 'Cancel',
  handleSubmit,
}) => (
  <Root>
    <IconButton icon="x" onClick={onClose} right={5} secondary top={5} />
    <H2>{title}</H2>
    {subtitle && <Text secondary>{subtitle}</Text>}
    <Fragment>
      <Row alignItems="center" mb={2}>
        <Item mr={1} vertical>
          <Label required>Email</Label>
          <ValidatedField
            component={TextField}
            name="email"
            validate={[required]}
          />
        </Item>
        <ModalMenuItem ml={1} vertical>
          <Label required>Role</Label>
          <ValidatedField
            component={input => (
              <Menu options={roles} {...input} placeholder="Please select" />
            )}
            name="role"
            validate={[required]}
          />
        </ModalMenuItem>
      </Row>

      <Row mb={2}>
        <Item mr={1} vertical>
          <Label>First Name</Label>
          <ValidatedField component={TextField} name="firstName" />
        </Item>
        <Item ml={1} vertical>
          <Label>Last Name</Label>
          <ValidatedField component={TextField} name="lastName" />
        </Item>
      </Row>

      <RowOverrideAlert alignItems="center" mb={2}>
        <ModalMenuItem mr={1} vertical>
          <Label>Title</Label>
          <ValidatedField
            component={input => (
              <Menu options={titles} {...input} placeholder="Please select" />
            )}
            name="title"
          />
        </ModalMenuItem>
        <ModalMenuItem ml={1} vertical>
          <Label>Country</Label>
          <ValidatedField
            component={input => (
              <Menu
                options={countries}
                {...input}
                placeholder="Please select"
              />
            )}
            name="country"
          />
        </ModalMenuItem>
      </RowOverrideAlert>

      <Row mb={2}>
        <Item vertical>
          <Label>Affiliation</Label>
          <ValidatedField component={TextField} name="affiliation" />
        </Item>
      </Row>
    </Fragment>

    <Buttons isFetching={isFetching}>
      {isFetching ? (
        <Spinner size={3} />
      ) : (
        <Fragment>
          <Button onClick={onClose}>{cancelText}</Button>
          <Button onClick={handleSubmit} primary>
            {confirmText}
          </Button>
        </Fragment>
      )}
    </Buttons>
  </Root>
)

const setInitialRole = a => {
  if (get(a, 'handlingEditor')) {
    return 'handlingEditor'
  }
  if (get(a, 'editorInChief')) {
    return 'editorInChief'
  }
  return 'author'
}

export default compose(
  withHandlers({
    onConfirm: ({ onConfirm, ...props }) => () => {
      if (onConfirm && typeof onConfirm === 'function') {
        onConfirm(props)
      }
    },
    onClose: ({ onCancel, ...props }) => () => {
      if (onCancel && typeof onCancel === 'function') {
        onCancel(props)
      }
      props.hideModal()
    },
  }),
  withProps(({ user }) => ({
    initialValues: {
      ...user,
      role: setInitialRole(user),
    },
  })),
  reduxForm({
    form: 'add-edit',
  }),
  setDisplayName('FormModal'),
)(FormModal)

// #region styles
const Root = styled.div`
  align-items: center;
  background: ${th('colorBackground')};
  border: ${th('borderWidth')} ${th('borderStyle')} transparent;
  border-radius: ${th('borderRadius')};
  box-shadow: ${th('boxShadow')};
  display: flex;
  flex-direction: column;
  position: relative;
  padding: calc(${th('gridUnit')} * 5);
  width: calc(${th('gridUnit')} * 60);

  ${H2} {
    margin: 0 0 ${th('gridUnit')} 0;
    text-align: center;
  }
`

const Buttons = styled.div`
  display: flex;
  justify-content: ${props => (props.isFetching ? 'center' : 'space-evenly')};
  margin-top: calc(${th('gridUnit')} * 3);
  width: 70%;
`
// #endregion
