const config = require('config')
const { get } = require('lodash')

const {
  User,
  Email,
  services,
  Fragment,
} = require('pubsweet-component-helper-service')
const { getEmailCopy } = require('./emailCopy')

const unsubscribeSlug = config.get('unsubscribe.url')
const { name: journalName, staffEmail } = config.get('journal')

module.exports = {
  async sendNotifications({
    isEQA,
    agree,
    baseUrl,
    collection,
    comments = '',
    User: UserModel,
    Fragment: FragmentModel,
  }) {
    const fragment = await FragmentModel.find(collection.fragments[0])
    const fragmentHelper = new Fragment({ fragment })
    const parsedFragment = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })

    const {
      activeAuthors: authors,
      submittingAuthor,
    } = await fragmentHelper.getAuthorData({
      UserModel,
    })

    const titleText = `manuscript titled "${parsedFragment.title}" by ${
      submittingAuthor.firstName
    } ${submittingAuthor.lastName}`
    const subjectBaseText = `${collection.customId}: Manuscript`
    const { customId } = collection
    const userHelper = new User({ UserModel })
    const subject = `${subjectBaseText} ${
      agree ? '' : 'Not '
    }Passed Technical Checks`

    const email = new Email({
      type: 'user',
      fromEmail: `${journalName} <${staffEmail}>`,
      content: {
        subject,
        signatureName: 'EQA Team',
        ctaLink: services.createUrl(
          baseUrl,
          `/projects/${collection.id}/versions/${fragment.id}/details`,
        ),
        ctaText: 'MANUSCRIPT DETAILS',
        unsubscribeLink: baseUrl,
      },
    })

    if (agree && isEQA) {
      email.content.signatureName = await userHelper.getEiCName()

      sendHandlingEditorEmail({
        email,
        baseUrl,
        titleText,
        subjectBaseText,
        handlingEditor: get(collection, 'handlingEditor', {}),
      })
      sendSubmittedReviewersEmail({
        email,
        baseUrl,
        titleText,
        UserModel,
        fragmentHelper,
        subjectBaseText,
      })

      sendAuthorsEmail({
        email,
        baseUrl,
        titleText,
        subjectBaseText,
        fragmentAuthors: authors,
      })
    } else {
      sendEditorsEmail({
        email,
        agree,
        customId,
        comments,
        titleText,
        userHelper,
      })
    }
  },
}

const sendEditorsEmail = async ({
  email,
  agree,
  titleText,
  customId,
  userHelper,
  comments = '',
}) => {
  email.content.subject = `${customId}: New manuscript submitted`
  const emailType = agree
    ? 'eqs-manuscript-accepted'
    : 'eqa-manuscript-returned-to-eic'

  const editors = (await userHelper.getEditorsInChief()).map(eic => ({
    ...eic,
    ...getEmailCopy({
      emailType,
      titleText,
      comments,
    }),
  }))

  editors.forEach(eic => {
    email.toUser = {
      email: eic.email,
      name: `${eic.firstName} ${eic.lastName}`,
    }
    const { html, text } = email.getBody({
      body: {
        paragraph: eic.paragraph,
        hasLink: eic.hasLink,
        hasIntro: eic.hasIntro,
        hasSignature: eic.hasSignature,
      },
    })
    email.sendEmail({ html, text })
  })
}

const sendHandlingEditorEmail = ({
  email,
  baseUrl,
  titleText,
  handlingEditor,
  subjectBaseText,
}) => {
  email.content.subject = `${subjectBaseText} Decision`
  const emailType = 'he-manuscript-published'

  email.toUser = {
    email: handlingEditor.email,
    name: handlingEditor.name,
  }
  email.content.unsubscribeLink = services.createUrl(baseUrl, unsubscribeSlug, {
    id: handlingEditor.id,
  })

  const { html, text } = email.getBody({
    body: getEmailCopy({
      emailType,
      titleText,
    }),
  })
  email.sendEmail({ html, text })
}

const sendSubmittedReviewersEmail = async ({
  email,
  baseUrl,
  titleText,
  UserModel,
  fragmentHelper,
  subjectBaseText,
}) => {
  email.content.subject = `${subjectBaseText} Decision`

  const reviewers = (await fragmentHelper.getReviewers({
    UserModel,
    type: 'submitted',
  })).map(rev => ({
    ...rev,
    ...getEmailCopy({
      emailType: 'submitted-reviewers-after-publish',
      titleText,
    }),
  }))

  reviewers.forEach(reviewer => {
    email.toUser = {
      email: reviewer.email,
      name: `${reviewer.firstName} ${reviewer.lastName}`,
    }
    email.content.unsubscribeLink = services.createUrl(
      baseUrl,
      unsubscribeSlug,
      {
        id: reviewer.id,
      },
    )
    const { html, text } = email.getBody({
      body: { paragraph: reviewer.paragraph, hasLink: reviewer.hasLink },
    })
    email.sendEmail({ html, text })
  })
}

const sendAuthorsEmail = ({
  email,
  baseUrl,
  titleText,
  subjectBaseText,
  fragmentAuthors,
}) => {
  const emailType = 'author-manuscript-published'
  email.content.subject = `${subjectBaseText} Published`

  const authors = fragmentAuthors.map(author => ({
    ...author,
    ...getEmailCopy({
      emailType,
      titleText,
    }),
  }))

  authors.forEach(author => {
    email.toUser = {
      email: author.email,
      name: `${author.firstName} ${author.lastName}`,
    }
    email.content.unsubscribeLink = services.createUrl(
      baseUrl,
      unsubscribeSlug,
      {
        id: author.id,
      },
    )
    const { html, text } = email.getBody({
      body: {
        paragraph: author.paragraph,
        hasLink: author.hasLink,
      },
    })
    email.sendEmail({ html, text })
  })
}
